<?php

/**
 * @package     Prieco.Modules
 * @subpackage  mod_cbrandomuser - This module shows a random user avatar.
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * ModcbrandomuserHelper
 *
 * @package     Prieco.Modules
 * @subpackage  mod_cbrandomuser
 * @since       1.0
*/
class ModcbrandomuserHelper
{
	const NO_SELECTION		= '- select entry -';

	/**
	 * getRandomUser
	 *
	 * @param   array  $params  the params
	 *
	 * @return  array
	 *
	 * @since   1.0
	 */
	public function getRandomUser($params)
	{
		$_CB_database = JFactory::getDBO();

		$user_format 			= (int) $params->get('user_format', 0);
		$avatar_type			= (int) $params->get('avatar_type', 0);
		$show_without_avatar	= (int) $params->get('show_without_avatar', 1);

		$condition_enabled		= (int) $params->get('condition_enabled', 0);
		$condition_field_1		= $params->get('condition_field_1', modcbrandomuserHelper::NO_SELECTION);
		$condition_operator_1 	= (int) $params->get('condition_operator_1', 0);
		$condition_value_1		= $params->get('condition_value_1', '');
		$condition_field_2		= $params->get('condition_field_2', modcbrandomuserHelper::NO_SELECTION);
		$condition_operator_2 	= (int) $params->get('condition_operator_2', 0);
		$condition_value_2		= $params->get('condition_value_2', '');
		$condition_logical		= (int) $params->get('condition_logical', 0);

		$add_fields_enabled		= (int) $params->get('add_fields_enabled', 0);
		$add_fields_type		= (int) $params->get('add_fields_type', 0);
		$add_fields_alignment	= (int) $params->get('add_fields_alignment', 0);

		$add_fields				=	array(
				$params->get('add_field_1', modcbrandomuserHelper::NO_SELECTION),
				$params->get('add_field_2', modcbrandomuserHelper::NO_SELECTION),
				$params->get('add_field_3', modcbrandomuserHelper::NO_SELECTION),
				$params->get('add_field_4', modcbrandomuserHelper::NO_SELECTION),
				$params->get('add_field_5', modcbrandomuserHelper::NO_SELECTION)
		);

		$avatar_image_url 	= '';
		$user_name			= '';
		$randuser			= null;
		$condition_where	= '';
		$avatar_where		= '';

		// Tablenames
		$table_comprofiler			= '#__comprofiler';
		$table_users				= '#__users';
		$table_comprofiler_fields	= '#__comprofiler_fields';

		// Get excluded users
		$filter_userids = trim($params->get('filter_userids', ""));

		if ($filter_userids != '' )
		{
			$tmp_uids = explode(',', $filter_userids);

			foreach ($tmp_uids as $ku => $uu )
			{
				$tmp_uids[$ku] = (int) $uu;
			}

			$filter_userids = ' AND u.id not in (' . implode(',', $tmp_uids) . ')';
		}

		// Check for avatar condition
		if (0 == $show_without_avatar)
		{
			$avatar_where = ' AND b.avatarapproved AND b.avatar ';
		}

		// Create conditions
		if ($condition_enabled && ($condition_field_1 != modcbrandomuserHelper::NO_SELECTION))
		{
			// First condition
			$operator_1			= modcbrandomuserHelper::getConditionOperator($condition_operator_1);
			$fieldAndValue_1	= modcbrandomuserHelper::getFieldAndValue($condition_field_1, $condition_value_1, $table_comprofiler);

			$condition_where = ' AND ' . $fieldAndValue_1[0] . ' ' . $operator_1 . ' ' . $fieldAndValue_1[1];

			// Second condition
			if (($condition_logical > 0) && ($condition_field_2 != modcbrandomuserHelper::NO_SELECTION))
			{
				switch ($condition_logical)
				{
					case 1:
						$logical_op = ' AND ';
						break;
					case 2:
						$logical_op = ' OR ';
						break;
				}

				$operator_2			= modcbrandomuserHelper::getConditionOperator($condition_operator_2);
				$fieldAndValue_2	= modcbrandomuserHelper::getFieldAndValue($condition_field_2, $condition_value_2, $table_comprofiler);

				$condition_where .= $logical_op . $fieldAndValue_2[0] . ' ' . $operator_2 . ' ' . $fieldAndValue_2[1];
			}
		}

		// Additional fields: get values
		// Use in select
		$selected_fields_select = 'u.' . $_CB_database->quoteName('id');

		// Use in where clause
		$selected_fields_where = '';

		if ($add_fields_enabled)
		{
			// Construct selected fields
			foreach ($add_fields as $add_field)
			{
				if (modcbrandomuserHelper::NO_SELECTION != $add_field)
				{
					$selected_fields_select .= ', ';
					$selected_fields_select .= 'b.' . $_CB_database->quoteName($add_field);

					if ('' != $selected_fields_where)
					{
						$selected_fields_where .= ', ';
					}

					$selected_fields_where .= $_CB_database->Quote($add_field);
				}
			}
		}

		// Get a random user and his field values
		$query = 'SELECT ' . $selected_fields_select
		. ' FROM ' . $_CB_database->quoteName($table_users) . ' u, ' . $_CB_database->quoteName($table_comprofiler) . ' b '
				. ' WHERE u.id = b.id AND b.approved AND u.block = 0 '
						. $avatar_where
						. $condition_where
						. $filter_userids
						. ' ORDER BY RAND() '
								. ' LIMIT 1';

		$_CB_database->setQuery($query);
		$user_fields = $_CB_database->loadAssoc();
		$uid = $user_fields['id'];

		if ($uid)
		{
			$cbUser =& CBuser::getInstance($uid);

			if (!empty($cbUser))
			{
				// Avatar image: changed to my own implemenation because cbUserAvatar function does not alltimes work! (broken path...)
				$baseurl = JUri::root();
				$avatar = $cbUser->getUserData()->avatar;

				if ((strlen($avatar) > 0) && (1 == (int) $cbUser->getUserData()->avatarapproved))
				{
					if ((0 == $avatar_type) && (false === strpos($avatar, 'gallery/')))
					{
						$avatar = 'tn' . $avatar;
					}

					$avatar_image_url  = $baseurl . 'images/comprofiler/' . $avatar;
				}
				else
				{
					$avatar_image_url  = $baseurl . 'components/com_comprofiler/images/english/tnnophoto.jpg';
				}

				// User name
				switch ($user_format)
				{
					case 0:
						$user_name = $cbUser->getUserData()->name;
						break;
					case 1:
						$user_name = $cbUser->getUserData()->username;
						break;
					default:
						$user_name = $cbUser->getUserData()->username;
				}

				// Additional fields: get field names for display
				$field_names = null;

				if ($add_fields_enabled && ('' != $selected_fields_where))
				{
					$query = 'SELECT ' . $_CB_database->quoteName('name') . ', ' . $_CB_database->quoteName('title')
					. ' FROM ' . $_CB_database->quoteName($table_comprofiler_fields)
					. ' WHERE ' . $_CB_database->quoteName('name') . ' IN (' . $selected_fields_where . ')'
							. ' ORDER BY FIELD(' . $_CB_database->quoteName('name') . ', ' . $selected_fields_where . ')';

					$_CB_database->setQuery($query);
					$field_names = $_CB_database->loadAssocList();
				}

				$randuser = array('user_fields' => $user_fields, 'field_names' => $field_names, 'name' => $user_name, 'avatar' => $avatar_image_url);
			}
		}

		return $randuser;
	}

	/**
	 * getConditionOperator
	 *
	 * @param   string  $condition_operator  the params
	 *
	 * @return  string
	 *
	 * @since   1.0
	 */
	private function getConditionOperator($condition_operator)
	{
		switch ($condition_operator)
		{
			// Equal
			case 0:
				$operator = '=';
				break;

				// Not equal
			case 1:
				$operator = '!=';
				break;

				// Less than
			case 2:
				$operator = '<';
				break;

				// Greater
			case 3:
				$operator = '>';
				break;

				// Less than or equal to
			case 4:
				$operator = '<=';
				break;

				// Greater than or equal to
			case 5:
				$operator = '>=';
				break;

				// In
			case 6:
				$operator = 'IN';
				break;

				// Not in
			case 7:
				$operator = 'NOT IN';
				break;

				// Like
			case 8:
				$operator = 'LIKE';
				break;
			default:
				$operator = '=';
		}

		return $operator;
	}

	/**
	 * Get type of condition field: this is tricky and not database indipendent! (mixed usage of orginal Joomla DB class and PHP mysql function needed
	 *
	 * @param   string  $condition_field  the params
	 * @param   string  $condition_value  the params
	 * @param   object  $table            the params
	 *
	 * @return  array
	 *
	 * @since   1.0
	 */
	private function getFieldAndValue($condition_field, $condition_value, $table)
	{
		$_CB_database = JFactory::getDBO();

		$type = modcbrandomuserHelper::getFieldType($condition_field, $table);

		switch ($type)
		{
			case 'int':
				$value = (int) $condition_value;
				$field = 'b.' . $_CB_database->quoteName($condition_field);
				break;
			case 'date':
			case 'datetime':
				$value = 'Date(' . $_CB_database->Quote($condition_value) . ')';
				$field = 'Date(' . 'b.' . $_CB_database->quoteName($condition_field) . ')';
				break;

				// String in most cases
			default:
				$value = $_CB_database->Quote($condition_value);
				$field = 'b.' . $_CB_database->quoteName($condition_field);
		}

		return array($field, $value);
	}

	/**
	 * getFieldType
	 *
	 * @param   string  $fieldname  the params
	 * @param   object  $table      the params
	 *
	 * @return  array
	 *
	 * @since   1.0
	 */
	public function getFieldType($fieldname, $table)
	{
		$_CB_database = JFactory::getDBO();

		$query = 'SELECT ' . $_CB_database->quoteName($fieldname)
		. ' FROM ' . $_CB_database->quoteName($table) . ' LIMIT 1';
		$_CB_database->setQuery($query);
		$result = $_CB_database->execute();
		$type = @mysql_field_type($result, 0);

		return $type;
	}
}
